'use strict';

Object.defineProperty(exports, "__esModule", {
  value: true
});
exports.ASYNC_META = undefined;
exports.default = createActionAsync;

var _reduxAct = require('redux-act');

var _lodash = require('lodash.defaults');

var _lodash2 = _interopRequireDefault(_lodash);

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { default: obj }; }

var ASYNC_META = exports.ASYNC_META = {
  REQUEST: "REQUEST",
  OK: "OK",
  ERROR: "ERROR"
};

var defaultOption = {
  request: {
    metaReducer: function metaReducer() {
      return ASYNC_META.REQUEST;
    }
  },
  ok: {
    metaReducer: function metaReducer() {
      return ASYNC_META.OK;
    }
  },
  error: {
    metaReducer: function metaReducer() {
      return ASYNC_META.ERROR;
    }
  }
};

function createActionAsync(description, api) {
  var options = arguments.length <= 2 || arguments[2] === undefined ? defaultOption : arguments[2];

  (0, _lodash2.default)(options, defaultOption);
  var actions = {
    request: (0, _reduxAct.createAction)(description + '_REQUEST', options.request.payloadReducer, options.request.metaReducer),
    ok: (0, _reduxAct.createAction)(description + '_OK', options.ok.payloadReducer, options.ok.metaReducer),
    error: (0, _reduxAct.createAction)(description + '_ERROR', options.error.payloadReducer, options.error.metaReducer)
  };

  var actionAsync = function actionAsync() {
    for (var _len = arguments.length, args = Array(_len), _key = 0; _key < _len; _key++) {
      args[_key] = arguments[_key];
    }

    return function (dispatch) {
      dispatch(actions.request.apply(actions, args));
      return api.apply(undefined, args).then(function (response) {
        dispatch(actions.ok({
          request: args,
          response: response
        }));
      }).catch(function (error) {
        var errorOut = {
          request: args,
          error: error
        };
        dispatch(actions.error(errorOut));
        if (options.rethrow) throw errorOut;
      });
    };
  };
  actionAsync.request = actions.request;
  actionAsync.ok = actions.ok;
  actionAsync.error = actions.error;
  return actionAsync;
};